Exception: GeneValidator::SequenceTypeError

Inherits:
Exception
  • Object
show all
Defined in:
lib/genevalidator/exceptions.rb

Overview

Exception raised when the command line type argument does not corrsepond to the type of the sequences in the fasta file

Instance Method Summary collapse

Instance Method Details

#to_sObject



10
11
12
13
# File 'lib/genevalidator/exceptions.rb', line 10

def to_s
  "\nSequence Type error: Possible cause include that the blast output" \
  " was not obtained against a protein database.\n"
end