Exception: GeneValidator::SequenceTypeError
- Inherits:
-
Exception
- Object
- Exception
- GeneValidator::SequenceTypeError
- Defined in:
- lib/genevalidator/exceptions.rb
Overview
Exception raised when the command line type argument does not corrsepond to the type of the sequences in the fasta file